from django.core.management.base import BaseCommand

from okrand import (
    update_mo_files,
    update_po_files,
)


class Command(BaseCommand):
    help = 'Okrand internationalization'

    # def add_arguments(self, parser):
    #     parser.add_argument('interactive', type=bool)

    def handle(self, *args, **options):
        update_po_files()
        update_mo_files()

#
# def main_interactive():
#     # TODO: CLI interactive and non-interactive modes
#     # TODO: suggest best match: https://stackoverflow.com/questions/6690739/high-performance-fuzzy-string-comparison-in-python-use-levenshtein-or-difflib
#
#     from textual.app import (
#         App,
#         ComposeResult,
#     )
#     from textual.widgets import (
#         ListView,
#         ListItem,
#         Label,
#         Header,
#     )
#
#     result = update_po_files()
#
#     if not result.newly_obsolete_strings:
#         return
#
#     class Okrand(App):
#
#         # CSS_PATH = "list_view.css"
#
#         def compose(self) -> ComposeResult:
#             yield Header()
#             yield Label(f'{len(result.newly_obsolete_strings)} left')
#             yield Label('Select which string matches this string:')
#             yield Label(repr(result.newly_obsolete_strings[0]))
#             yield ListView(
#                 ListItem(Label('< none of these match >')),
#                 *[
#                     ListItem(Label(repr(x)))
#                     for x in result.new_strings
#                 ]
#             )
#
#         def on_ready(self):
#             self.screen.focus_next()
#
#     app = Okrand()
#     app.run()
#
#     # result = update_po_files()
#     # if not result.newly_obsolete_strings:
#     #     break
#     #
#     #
#     #
#     # print('new:')
#     # for i, x in enumerate(result.new_strings):
#     #     print('    ', repr(x))
#     #
#     # print()
#     #
#     # print('newly obsolete:')
#     # for x in result.newly_obsolete_strings:
#     #     print('    ', repr(x))
#



from django.apps import (
    AppConfig,
    apps,
)
from django.utils.functional import Promise
from django.utils.translation import gettext_lazy

from okrand import get_conf


def get_args_from_proxy(proxy):
    args = getattr(proxy, '_args', None)
    if args is None:
        args = getattr(proxy, '_proxy____args', None)
    return args


def get_original_string_from_proxy(proxy):
    args = get_args_from_proxy(proxy)
    return args[0]


def upgrade_verbose_name(obj):
    original = obj.verbose_name
    if isinstance(original, Promise):
        obj.verbose_name._okrand_original_string = get_original_string_from_proxy(original)
        assert not isinstance(obj.verbose_name._okrand_original_string, Promise)
        return

    obj.verbose_name = gettext_lazy(original)
    obj.verbose_name._okrand_original_string = original
    assert not isinstance(obj.verbose_name._okrand_original_string, Promise)


def upgrade_plural(obj):
    original = obj.verbose_name_plural
    if isinstance(original, Promise):
        args = get_args_from_proxy(original)
        # if a Django autogenerated plural string
        if len(args) == 2 and args[0] == '{}s':
            original = str(original)
        else:
            assert len(args) == 1
            obj.verbose_name_plural._okrand_original_string = args[0]
            assert not isinstance(obj.verbose_name_plural._okrand_original_string, Promise)
            return

    new = gettext_lazy(original)
    obj.verbose_name_plural = new
    obj.verbose_name_plural._okrand_original_string = original
    assert not isinstance(obj.verbose_name_plural._okrand_original_string, Promise)


class OkrandConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'okrand'

    def ready(self):
        if get_conf('django_model_upgrade') in ('1', 'true'):
            for model in apps.get_models():
                upgrade_verbose_name(model._meta)
                upgrade_plural(model._meta)

                for field in model._meta._get_fields(reverse=False):
                    upgrade_verbose_name(field)



#!/usr/bin/env python
import os
import re
from io import open

from setuptools import (
    Command,
    setup,
)

with open('README.rst', encoding='utf8') as f:
    readme = f.read()


def read_reqs(name):
    with open(os.path.join(os.path.dirname(__file__), name), encoding='utf8') as f:
        return [line for line in f.read().split('\n') if line and not line.strip().startswith('#')]


def read_version():
    with open(os.path.join('okrand', '__init__.py'), encoding='utf8') as f:
        m = re.search(r'''__version__\s*=\s*['"]([^'"]*)['"]''', f.read())
        if m:
            return m.group(1)
        raise ValueError("couldn't find version")


class Tag(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from subprocess import call

        version = read_version()
        errno = call(['git', 'tag', '--annotate', version, '--message', 'Version %s' % version])
        if errno == 0:
            print("Added tag for version %s" % version)
        raise SystemExit(errno)


class ReleaseCheck(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from subprocess import check_output, CalledProcessError

        try:
            tag = check_output(['git', 'describe', 'HEAD']).strip().decode('utf8')
        except CalledProcessError:
            tag = ''
        version = read_version()
        if tag != version:
            print('Missing %s tag on release' % version)
            raise SystemExit(1)

        current_branch = check_output(['git', 'rev-parse_query_string', '--abbrev-ref', 'HEAD']).strip().decode('utf8')
        if current_branch != 'master':
            print('Only release from master')
            raise SystemExit(1)

        print("Ok to distribute files")


# NB: _don't_ add namespace_packages to `setup()`, it'll break
#     everything using imp.find_module
setup(
    name='okrand',
    version=read_version(),
    description='Okrand is an internationalization/translation tool for Django',
    long_description=readme,
    author='Anders Hovmöller',
    author_email='boxed@killingar.net',
    url='https://github.com/boxed/okrand',
    packages=['okrand'],
    include_package_data=True,
    install_requires=read_reqs('requirements.txt'),
    license="BSD",
    zip_safe=False,
    keywords='okrand',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
    ],
    test_suite='tests',
    cmdclass={'tag': Tag, 'release_check': ReleaseCheck},
)
